# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from infrae.rest.components import queryRESTComponent
from infrae.rest.components import REST, RESTWithTemplate, lookupREST
from infrae.rest.interfaces import IRESTMethodPublishedEvent

__all__ = [
    'REST', 'lookupREST', 'RESTWithTemplate', 'IRESTMethodPublishedEvent',
    'queryRESTComponent']




# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from OFS.Folder import Folder
from five import grok
from infrae.rest import RESTWithTemplate


class LocalStatic(grok.DirectoryResource):
    """Local static folder is registered by hand here.
    """
    grok.path('static')
    grok.name('infrae.rest.tests.grok.template_fixture')


class FormTemplate(RESTWithTemplate):
    grok.context(Folder)
    grok.name('form')

    def value(self):
        return '42'

    def GET(self):
        return self.template.render(self)


class WidgetTemplate(RESTWithTemplate):
    grok.adapts(FormTemplate, Folder)
    grok.name('widget')

    def value(self):
        return '51'

    def GET(self):
        return self.template.render(self)


# Let's use an inline template here
widgettemplate = grok.PageTemplate('<span tal:content="rest/value"></span>')



# -*- coding: utf-8 -*-
# Copyright (c) 2012-2013 Infrae. All rights reserved.
# See also LICENSE.txt
"""

  Let's first grok our test first:

   >>> grok('infrae.rest.tests.grok.template_fixture')

  Now, if we have a folder we can use our REST component:

    >>> root = getRootFolder()
    >>> root.manage_addProduct['OFS'].manage_addFolder('folder', 'Folder')
    >>> from five.localsitemanager import make_site
    >>> make_site(root.folder)

  We can now fetch the component with the template:

    >>> print http('GET /folder/++rest++form HTTP/1.0')
    HTTP/1.0 200 OK
    Content-Length: 168
    Content-Type: text/html;charset=utf-8
    <BLANKLINE>
    <form>
       <h1>42</h1>
      <link rel="stylesheet" type="text/css"
            href="http://localhost/folder/@@/infrae.rest.tests.grok.template_fixture/style.css" />
    </form>
    <BLANKLINE>

  And the nested one as well:

    >>> print http('GET /folder/++rest++form/widget HTTP/1.0')
    HTTP/1.0 200 OK
    Content-Length: 15
    Content-Type: text/html;charset=utf-8
    <BLANKLINE>
    <span>51</span>


"""



# -*- coding: utf-8 -*-
# Copyright (c) 2010-2013 Infrae. All rights reserved.
# See also LICENSE.txt

from App.class_init import InitializeClass as initializeClass
from Products.Five.security import protectName

from five import grok
from grokcore.view.meta.views import TemplateGrokker
import martian

from infrae.rest.components import REST, ALLOWED_REST_METHODS
from infrae.rest.components import RESTWithTemplate


class RESTSecurityGrokker(martian.ClassGrokker):
    """Grok REST views.
    """
    martian.component(REST)
    martian.priority(100)
    martian.directive(grok.require, name='permission')

    def execute(self, factory, permission, config, **kw):
        """Register the REST component as a view on the IREST layer.
        """
        # Set name. The priority of the grokker must be higher than 0
        # to be call before the component is registered by
        # zeam.component.
        if not grok.name.bind().get(factory):
            grok.name.set(factory, factory.__name__.lower())

        if permission is None:
            permission = 'zope.Public'

        for method in ALLOWED_REST_METHODS:
            if hasattr(factory, method):
                config.action(
                    discriminator = ('five:protectName', factory, method),
                    callable = protectName,
                    args = (factory, method, permission))

        config.action(
            discriminator = ('five:initialize:class', factory),
            callable = initializeClass,
            args = (factory,))
        return True


class RESTWithTemplateGrokker(TemplateGrokker):
    martian.component(RESTWithTemplate)

    def has_render(self, factory):
        return False

    def has_no_render(self, factory):
        return False

